//******************************************************************************
//  MSPFW42x ScanInterface SIFCLK Adjustment
//
//  Description:
//  The Timing State Machine (TSM) of the MSP430FW42x's Scan Interface has an
//  own oscillator. It is called SIFOSC and it can be chosen as the source for
//  the Timing State Machine (SIFCLK). There is a counter that can be used to
//  adjust the SIFOSC frequency based on a reference clock (ACLK=32kHz). The
//  functions of this file uses the integrated parts for adjustment of the
//  SIFOSC frequency.
//
//  Christian Hernitscheck
//  Texas Instruments Deutschland GmbH
//  April 2006
//  Built with IAR Embedded Workbench Version: 3.30A
//******************************************************************************
// THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
// REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY,
// INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR
// COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE.
// TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET
// POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY
// INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
// YOUR USE OF THE PROGRAM.
//
// IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY
// THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT
// OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.
// EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF
// REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS
// OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF
// USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
// AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF
// YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS
// (U.S.$500).
//
// Unless otherwise stated, the Program written and copyrighted
// by Texas Instruments is distributed as "freeware".  You may,
// only under TI's copyright in the Program, use and modify the
// Program without any charge or restriction.  You may
// distribute to third parties, provided that you transfer a
// copy of this license to the third party and the third party
// agrees to these terms by its first use of the Program. You
// must reproduce the copyright notice and any other legend of
// ownership on each copy or partial copy, of the Program.
//
// You acknowledge and agree that the Program contains
// copyrighted material, trade secrets and other TI proprietary
// information and is protected by copyright laws,
// international copyright treaties, and trade secret laws, as
// well as other intellectual property laws.  To protect TI's
// rights in the Program, you agree not to decompile, reverse
// engineer, disassemble or otherwise translate any object code
// versions of the Program to a human-readable form.  You agree
// that in no event will you alter, remove or destroy any
// copyright notice included in the Program.  TI reserves all
// rights not specifically granted under this license. Except
// as specifically provided herein, nothing in this agreement
// shall be construed as conferring by implication, estoppel,
// or otherwise, upon you, any license or other right under any
// TI patents, copyrights or trade secrets.
//
// You may not use the Program in non-TI devices.
//
//******************************************************************************
#include "msp430xw42x.h"

void Calibrate_SIF_Oscillator(void);
unsigned char Measure_SIFOSC(void);
int IncSIFCLKFQ(void);
int DecSIFCLKFQ(void);
unsigned char GetSIFCNT3(void);


unsigned int Debug[100];
int ptr;

/*---------------------------------------------------------------------------*/
void main(void)
{
  WDTCTL = WDTPW + WDTHOLD;         // disable Watchdog
  FLL_CTL0 |= XCAP10PF;             // Configure load caps

  P1OUT = 0x00;      // All port pins are outputs
  P1DIR = 0xFF;
  P2OUT = 0x00;
  P2DIR = 0xFF;
  P3OUT = 0x00;
  P3DIR = 0xFF;

  P2SEL |= 0x80;     // SIFCLK on pin P2.7
  P1SEL |= 0x20;     // ACLK on pin P1.5

  SIFCTL5 |= 0x07;   // SIFFNOM=1 (=> 1MHz)
                     // SIFCLKEN=1 -> Scan IF internal oscillator is used
  SIFCTL1 |= 0x01;   // Scan IF enabled

  ptr=0;             // ptr is used for debugging (pointer for debug array)

  while(1)
    Calibrate_SIF_Oscillator();
}

/*---------------------------------------------------------------------------*/

void Calibrate_SIF_Oscillator (void)
{ unsigned char Error;

  if (Measure_SIFOSC() < (unsigned char) 128)
  {                         // Frequency too high
    do
    {  Error = IncSIFCLKFQ();
    } while (Measure_SIFOSC() < 128);
    Error = DecSIFCLKFQ();
  }
  else
  {                         // Frequency too low
    do
    {   Error = DecSIFCLKFQ();
    } while (Measure_SIFOSC() > 128);
  }
  if (Error==1)
    P1OUT |= 0x80;              //--- set Error LED (=> Error handling has to
                                //--- be done here, e.g. return value)
}

unsigned char Measure_SIFOSC (void)
{ unsigned char result;
  int LoopCount;

  LoopCount=0;
  SIFCTL5 &= ~0x0003;           // setting of SIFCLKEN and
  SIFCTL5 |= 0x0003;            //     SIFCLKGON resets SIFCNT3
  while (GetSIFCNT3() == 0x01)
   {LoopCount=LoopCount+1;}     // wait till measurement is finished
  result = GetSIFCNT3();

  Debug[ptr]=SIFCTL5;           //--- These lines are just for debugging.
  ptr=ptr+1;                    //--- It allows to check the counter values.

  return result;
}

unsigned char GetSIFCNT3(void)
{ unsigned char result ;
  result = SIFCTL5 >> 8;
  result = result & 0x00FF;
  return result;
}


int IncSIFCLKFQ (void)      // Increase Frequency by 1 step
{ unsigned int result;
  unsigned char Error;
  unsigned int temp;

  Error = 0;
  result = SIFCTL5 >> 3;
  result = result & 0x000F;
  result = result + 1;
  if (result >= 16)
    Error = 1;
  result = result << 3;
  temp = SIFCTL5;
  temp &= ~0x0078;
  SIFCTL5 = temp | result;
  return Error;
}

int DecSIFCLKFQ (void)      // Decrease Frequency by 1 step
{ unsigned int result;
  unsigned char Error;
  unsigned int temp;

  Error = 0;
  result = SIFCTL5 >> 3;
  result = result & 0x000F;
  if (result == 0)
    Error = 1;
  result = result - 1;
  result = result << 3;
  temp = SIFCTL5;
  temp &= ~0x0078;
  SIFCTL5 = temp | result;
  return Error;
}
